"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const disposable_1 = require("@phosphor/disposable");
const apputils_1 = require("@jupyterlab/apputils");
const TOOLBAR_KERNEL_NAME_CLASS = 'jp-Toolbar-kernelName';
/**
 * Initialization data for the kernel_reconnector_extension extension.
 */
const extension = {
    id: 'fk_extension:kernel_reconnector_extension',
    autoStart: true,
    activate: (app) => {
        app.docRegistry.addWidgetExtension('Notebook', new KernelReconnectorExtension());
    }
};
class KernelReconnector extends apputils_1.ToolbarButton {
    constructor(session) {
        super({
            className: TOOLBAR_KERNEL_NAME_CLASS,
            onClick: () => {
                console.log("Reconnecting on user request");
                session.kernel.reconnect();
            }
        });
        this.previousIOPubTime = new Date().getTime();
        this.node.textContent = 'Reconnect Kernel';
        session.iopubMessage.connect(this._updateTimeStamp, this);
        // Set up polling.
        let reconnectTimer = setInterval(() => {
            // if (typeof document !== 'undefined' && document.hidden) {
            //   // Don't poll when nobody's looking.
            //   return;
            // }
            this._refreshRunning(session);
        }, 5000);
        reconnectTimer;
    }
    _delay(ms) {
        return new Promise(resolve => setTimeout(resolve, ms));
    }
    _updateTimeStamp() {
        return __awaiter(this, void 0, void 0, function* () {
            this.previousIOPubTime = new Date().getTime();
        });
    }
    _refreshRunning(session) {
        return __awaiter(this, void 0, void 0, function* () {
            let timeNow = new Date().getTime();
            if (((Math.abs(timeNow - this.previousIOPubTime) / (1000 * 60)) > 30)) {
                yield this._delay(5 * 1000);
                console.log("Reconnecting after 30 minutes of inactivity");
                session.kernel.reconnect();
            }
        });
    }
}
exports.KernelReconnector = KernelReconnector;
class KernelReconnectorExtension {
    /**
     * Create a new extension object.
     */
    createNew(panel, context) {
        let button = new KernelReconnector(panel.session);
        panel.toolbar.insertItem(9, 'reconnectKernel', button);
        return new disposable_1.DisposableDelegate(() => {
            button.dispose();
        });
    }
}
exports.KernelReconnectorExtension = KernelReconnectorExtension;
exports.default = extension;
